import {Component, OnInit} from '@angular/core';
import {InputOutputComponent} from "../input-output/input-output.component";
import {DebounceTime, GenerateDefaultJsonObjectString, MonacoJsonConfig, ReadOnlyMonacoCSharpConfig} from "../defaults";
import {BehaviorSubject, debounceTime} from "rxjs";
import {JtrService} from "../jtr.service";
import {MatFormField} from "@angular/material/form-field";
import {MatInputModule} from "@angular/material/input";

@Component({
  selector: 'app-json-path',
  standalone: true,
  imports: [
    InputOutputComponent,
    MatFormField,
    MatInputModule
  ],
  templateUrl: './json-path.component.html',
  styleUrl: './json-path.component.scss'
})
export class JsonPathComponent implements OnInit {
  $path: BehaviorSubject<string> = new BehaviorSubject<string>("$.*");
  $input: BehaviorSubject<string> = new BehaviorSubject<string>(GenerateDefaultJsonObjectString(2));
  inputOptions = MonacoJsonConfig;
  output: string = "";
  outputOptions = ReadOnlyMonacoCSharpConfig;

  constructor(private service: JtrService) {
  }

  ngOnInit(): void {
    this.$input
      .pipe(debounceTime(DebounceTime))
      .subscribe(input => this.update(input, this.$path.value));

    this.$path
      .pipe(debounceTime(DebounceTime))
      .subscribe(path => this.update(this.$input.value, path));

    this.update(this.$input.value, this.$path.value);
  }

  update(input: string, path: string): void {
    this.service
      .jsonPath(input, path)
      .subscribe({
        next: response => {
          console.log(response);
          this.output = response.body.result;
        },
        error: response => {
          console.log(response)
          if (response.status === 499) {
            this.output = response.error.detail;
            console.log(response.error.detail);
          }
        }
      });
  }

  handlePathChange($event: any): void {
    console.log($event);
    this.$path.next($event.target.value);
  }

  handleInputChange($event: any): void {
    console.log($event);
    this.$input.next($event);
  }
}
